<?php

namespace Soisy\PaymentMethod\Controller\Adminhtml\Callback;

use Magento\Backend\App\Action\Context;
use Magento\Framework\DB\TransactionFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Api\Data\OrderStatusHistoryInterface;
use Magento\Sales\Model\OrderRepository;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Email\Sender\OrderSender;
use Soisy\PaymentMethod\Log\Logger;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Sales\Api\OrderManagementInterface;
use Magento\Sales\Model\Service\InvoiceService;

class Index extends \Magento\Framework\App\Action\Action implements CsrfAwareActionInterface
{
    const ORDER_STASTUS_PAYMENT_CONFIRMED = 'payment_confirmed';

    /**
     * @var Order
     */
    protected $order;

    /**
     * @var OrderSender
     */
    protected $orderSender;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var OrderRepository
     */
    protected $orderRepository;

    /**
     * @var OrderManagementInterface
     */
    protected $orderManagementInterface;

    /**
     * @var InvoiceService
     */
    protected $invoiceService;

    /**
     * @var TransactionFactory
     */
    protected $transactionFactory;

    public function __construct(
        Context                  $context,
        Order                    $order,
        OrderSender              $orderSender,
        Logger                   $logger,
        OrderRepository          $orderRepository,
        OrderManagementInterface $orderManagementInterface,
        InvoiceService           $invoiceService,
        TransactionFactory       $transactionFactory
    )
    {
        parent::__construct($context);
        $this->order = $order;
        $this->orderSender = $orderSender;
        $this->logger = $logger;
        $this->orderRepository = $orderRepository;
        $this->orderManagementInterface = $orderManagementInterface;
        $this->invoiceService = $invoiceService;
        $this->transactionFactory = $transactionFactory;
    }

    public function execute()
    {
        try {
            $post = $this->getRequest()->getPostValue();

            $eventId = isset($post['eventId']) ? $post['eventId'] : '';
            $eventMessage = isset($post['eventMessage']) ? $post['eventMessage'] : '';
            $eventDate = isset($post['eventDate']) ? $post['eventDate'] : '';
            $orderToken = isset($post['orderToken']) ? $post['orderToken'] : '';
            $orderReference = isset($post['orderReference']) ? $post['orderReference'] : '';
            $amount = isset($post['amount']) ? $post['amount'] : '';

            if ($eventId == '' || $eventMessage == '' || $eventDate == '' || $orderToken == '' || $orderReference == '') {
                $this->logger->log('notice', 'CALLBACK: Malformed callback request');
                $this->_return400();
                return;
            }

            $order = $this->order->loadByIncrementId($orderReference);
            $orderId = $order->getId();

            if (is_null($order) || is_null($order->getId())) {
                $this->logger->log('notice', 'CALLBACK: Order ' . $orderReference . ' not found');
                $this->_return400();
                return;
            }

            if ($order->getPayment()->getMethod() != 'soisy') {
                $this->logger->log('notice', 'CALLBACK: Order ' . $orderReference . ' is not a Soisy order');
                $this->_return400();
                return;
            }

            $statusHistory = $this->_objectManager->create(OrderStatusHistoryInterface::class);
            $statusHistory->setComment('Soisy callback received: ' . json_encode($post));
            $statusHistory->setStatus($order->getStatus());
            $this->orderManagementInterface->addComment(
                $orderId,
                $statusHistory
            );
            $this->orderRepository->save($order);

            switch ($eventId) {
                case "LoanWasApproved":
                case "RequestCompleted":
                    if (!$order->getEmailSent()) {
                        $this->orderSender->send($order);
                    }
                    $statusHistory = $this->_objectManager->create(OrderStatusHistoryInterface::class);
                    $statusHistory->setComment('Soisy update: ' . $eventMessage);
                    $statusHistory->setStatus(Order::STATE_PENDING_PAYMENT);
                    $this->orderManagementInterface->addComment(
                        $orderId,
                        $statusHistory
                    );
                    $order->setState(Order::STATE_PENDING_PAYMENT);
                    $order->setStatus(Order::STATE_PENDING_PAYMENT);
                    break;
                case "LoanWasDisbursed":
                    $statusHistory = $this->_objectManager->create(OrderStatusHistoryInterface::class);
                    $statusHistory->setComment('Soisy update: ' . $eventMessage);
                    $statusHistory->setStatus($order->getStatus());
                    $this->orderManagementInterface->addComment(
                        $orderId,
                        $statusHistory
                    );
                    break;
                case "LoanWasVerified":
                    if (!$order->canInvoice()) {
                        throw new LocalizedException('Order cannot be invoiced');
                    }
                    $invoice = $this->invoiceService->prepareInvoice($order);

                    if (!$invoice->getTotalQty()) {
                        throw new LocalizedException('Cannot create an invoice without products.');
                    }

                    $invoice->setRequestedCaptureCase(\Magento\Sales\Model\Order\Invoice::CAPTURE_OFFLINE);

                    $invoice->register();
                    $transactionSave = $this->transactionFactory->create()
                        ->addObject($invoice)
                        ->addObject($invoice->getOrder());
                    $transactionSave->save();

                    $order->setState(Order::STATE_PROCESSING);
                    $order->setStatus(self::ORDER_STASTUS_PAYMENT_CONFIRMED);

                    $statusHistory = $this->_objectManager->create(OrderStatusHistoryInterface::class);
                    $statusHistory->setComment('Soisy update: ' . $eventMessage);
                    $statusHistory->setStatus($order->getStatus());
                    $this->orderManagementInterface->addComment(
                        $orderId,
                        $statusHistory
                    );
                    break;
                case "UserWasRejected":
                    $order->hold();
                    $statusHistory = $this->_objectManager->create(OrderStatusHistoryInterface::class);
                    $statusHistory->setComment('Soisy update: ' . $eventMessage);
                    $statusHistory->setStatus($order->getStatus());
                    $this->orderManagementInterface->addComment(
                        $orderId,
                        $statusHistory
                    );
                    break;
                default:
                    $this->logger->log("Event ID not recognised: " . $eventId);
            }

            $this->orderRepository->save($order);
        } catch (\Exception $e) {
            $this->logger->log($e->getMessage());
            if (isset($order) && !is_null($order) && $order->getId()) {
                $statusHistory = $this->_objectManager->create(OrderStatusHistoryInterface::class);
                $statusHistory->setComment('Soisy callback error: ' . $e->getMessage());
                $statusHistory->setStatus($order->getStatus());
                $this->orderManagementInterface->addComment(
                    $orderId,
                    $statusHistory
                );
                $this->orderRepository->save($order);
            }
        }
    }

    protected function _return400()
    {
        $this->getResponse()->setHttpResponseCode(400);
    }

    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }

    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
}